/*
    Copyright (c) 2012 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc> 

    This file is part of dss11-ledd.

    dss11-ledd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dss11-ledd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dss11-ledd. If not, see <http://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>

#include "ledd_common.h"
#include "sockutil.h"

static int send_command(ledd_actions command)
{
    int sockfd;
    struct addrinfo hints;
    struct addrinfo *server;
    unsigned char data[16];
    char port[8];

    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;
   
    snprintf(port, 8, "%d", DSS11_LEDD_PORT);
    if (getaddrinfo("127.0.0.1", port, &hints, &server) != 0)
    {
        fprintf(stderr, "Call to getaddrinfo failed!\n");
        return EXIT_FAILURE;
    }

    sockfd = socket(server->ai_family, server->ai_socktype,
                    server->ai_protocol);
    if (sockfd < 0)
    {
        fprintf(stderr, "Failed to create socket: %s\n", strerror(errno));
        freeaddrinfo(server); 
        return EXIT_FAILURE;
    }

    if (connect(sockfd, server->ai_addr, server->ai_addrlen) != 0)
    {
        fprintf(stderr, "Failed to connect to daemon: %s\n", strerror(errno));
        freeaddrinfo(server); 
        return EXIT_FAILURE;
    }

    memset(data, 0, sizeof(data));
    memcpy(data, &command, sizeof(command));
    
    ssize_t sent = sockwrite(sockfd, data, sizeof(command), 4);
    if (sent < (ssize_t)sizeof(command))
    {
        fprintf(stderr, "Error when sending command to daemon\n");
        freeaddrinfo(server); 
        return EXIT_FAILURE;
    }

    freeaddrinfo(server); 
    return EXIT_SUCCESS;
}

int main(int argc, char** argv)
{
    const char* parameter = NULL;
    if (argc < 2)
    {
        printf("Usage: %s <command>\n", argv[0]);
        return EXIT_FAILURE;
    }

    parameter = argv[1];
    ledd_actions command = ledd_string_to_action(parameter);
    if ((command < ledd_quit) || (command >= ledd_unhandled))
    {
        printf("Invalid command <%s>\n", parameter);
        return EXIT_FAILURE;
    }
    printf("Sending command: %s\n", parameter); 
    return send_command(command);
}
