/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of libcommchannel.

    libcommchannel is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libcommchannel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libcommchannel. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __CC_LIBCOMM_H__
#define __CC_LIBCOMM_H__

#include <boost/shared_ptr.hpp>
#include <stdint.h>

namespace CC {

class CommunicationDataInternal;
class CommunicationChannelServerInternal;
class CommunicationChannelClientInternal;

typedef struct CommunicationDataID
{
    int fd;
    struct timespec ts;
} CommunicationDataID_t;

class CommunicationData
{
public:
    CommunicationData(CommunicationDataID_t dataid, unsigned char *data, uint32_t size);
    
    // pass this id when sending the reply
    const CommunicationDataID_t getID() const;

    // get the pointer to the data
    const unsigned char *getDataPtr() const;

    // get the size of the data
    uint32_t getDataSize() const;

private:
    CommunicationData();
    boost::shared_ptr<CommunicationDataInternal> m_internal;
};

class CommunicationChannelCallback
{
public:
    // make sure not to block the callback as it is on the data receiving
    // thread
    virtual void messageReceived(boost::shared_ptr<CommunicationData> data) = 0;
    virtual ~CommunicationChannelCallback() {};
};

class CommunicationChannelServer
{
public:
    // open a local socket on a given port
    CommunicationChannelServer(int port, int maxconn = 10);
    // register callback to receive data
    void addCallback(CommunicationChannelCallback *cb);
    // remove previously registered calback
    void removeCallback(CommunicationChannelCallback *cb);
    // send reply
    // NOTE: only one reply can be sent in total, connection is closed
    //       after reply has been sent
    void sendReply(CommunicationDataID_t id, unsigned char *data,
                   uint32_t length);
private:
    boost::shared_ptr<CommunicationChannelServerInternal> m_internal;
};

class CommunicationChannelClient
{
public:
    // initialise class with a given target port
    CommunicationChannelClient(int port);

    // send data t target port and return the reply
    boost::shared_ptr<CommunicationData> sendData(unsigned char *data,
                                                  uint32_t size);
    boost::shared_ptr<CommunicationData> sendData(const std::string& data);

private:
    boost::shared_ptr<CommunicationChannelClientInternal> m_internal;
};

};

#endif
