/*
    Copyright (c) 2010, 2011, 2013 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of libcommchannel. Imported from dS Assistant project.

    libcommchannel is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libcommchannel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libcommchannel. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __CC_EXCEPTIONS_H__
#define __CC_EXCEPTIONS_H__

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

#define _CCException(msg) CC::Exception(msg, 0, __FILE__, __LINE__, __func__)
#define _CCExceptionWithCode(msg, code) \
                        CC::Exception(msg, code, __FILE__, __LINE__, __func__)

namespace CC
{

class ExceptionInternal;

class Exception
{
public:
    Exception(std::string message, int err, const char* file, int line,
              const char* function);
    Exception(std::string message);
    const std::string& getMessage() const;
    int getErrorCode() const;
    const std::vector<std::string> getStackTrace() const;
    void printStackTrace();

private:
    boost::shared_ptr<ExceptionInternal> m_internal;
};

}

#endif//__CC_EXCEPTIONS_H__
