/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of libcommchannel.

    libcommchannel is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libcommchannel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libcommchannel. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __COMMUNICATION_CHANNEL_H__
#define __COMMUNICATION_CHANNEL_H__

#include <pthread.h>
#include <boost/thread/mutex.hpp>
#include <boost/shared_ptr.hpp>
#include <list>

#include "include/commchannel.h"

namespace CC {

class CommunicationDataInternal
{
public:
    CommunicationDataInternal(CommunicationDataID_t dataid, unsigned char *data, uint32_t size);
    ~CommunicationDataInternal();
    // pass this id when sending the reply
    const CommunicationDataID_t getID() const;

    // get the pointer to the data
    const unsigned char *getDataPtr() const;

    // get the size of the data
    uint32_t getDataSize() const;

private:
    CommunicationDataInternal();
    CommunicationDataID_t m_id;
    unsigned char *m_data;
    uint32_t m_size;
};

/*
class CommunicationChannelCallback
{
public:
    virtual void messageReceived(boost::shared_ptr<CommunicationData> data) = 0;
    virtual ~CommunicationChannelCallback() {};
};
*/

class CommunicationChannelServerInternal
{
public:
    CommunicationChannelServerInternal(int port, int maxconn = 10);
    void addCallback(CommunicationChannelCallback *cb);
    void removeCallback(CommunicationChannelCallback *cb);
    void sendReply(CommunicationDataID_t id, unsigned char *data,
                   uint32_t length);
    ~CommunicationChannelServerInternal();

private:
    class Connection
    {
    public:
        Connection();
        int fd;
        struct timespec ts;
        bool replied;
        bool received;
        uint32_t size;
        unsigned char *data;
    };

    int m_port;
    int m_socket_fd;
    std::list<boost::shared_ptr<Connection> > m_connected_fds;
    int m_open_connections;
    int m_stop_flag;
    size_t m_max_connections;

    pthread_t m_listener_thread;

    boost::mutex m_callback_list_mutex;
    boost::mutex m_connections_mutex;
    std::list<CommunicationChannelCallback *> m_callbacks;

    void listenerThread();
    static void *staticThreadProc(void *arg);
    static int qsort_intcompare(const void *v1, const void *v2);

    void notifySubscribers(boost::shared_ptr<CommunicationData> data);
};

class CommunicationChannelClientInternal
{
public:
    CommunicationChannelClientInternal(int port);
    ~CommunicationChannelClientInternal();

    boost::shared_ptr<CommunicationData> sendData(unsigned char *data,
                                                  uint32_t size);

private:
    CommunicationChannelClientInternal();
    int m_port;
    int m_socket_fd;
    boost::mutex m_mutex;
};

};

#endif// __COMMUNICATION_CHANNEL_H__
