/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of libcommchannel. Imported from dS Assistant project.

    libcommchannel is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libcommchannel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libcommchannel. If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif


#ifdef HAVE_EXECINFO_H
    #include <execinfo.h>
    #include <stdlib.h>
#endif

#include <stdio.h>
#include "exceptions_int.h"


#define TRACE_TAG "_TRACE_"

namespace CC
{

ExceptionInternal::ExceptionInternal(std::string message, int err, const char* file, int line,
                     const char* function) : m_message(message),
                                             m_error_code(err),
                                             m_file(file),
                                             m_line(line)
{
#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    void *b[100];
    int size = backtrace(b, 100);

    m_stackTrace.reserve(size);

    char **s = backtrace_symbols(b, size);
    for(int i = 0; i < size; i++)
    {
        std::string trace(s[i]);
        m_stackTrace.push_back(trace);
    }
    if (s)
    {
        free(s);
    }
#endif

}

ExceptionInternal::ExceptionInternal(std::string message) : m_message(message), m_line(-1)
{
#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    void *b[100];
    int size = backtrace(b, 100);

    m_stackTrace.reserve(size);

    char **s = backtrace_symbols(b, size);
    for(int i = 0; i < size; i++)
    {
        std::string trace(s[i]);
        m_stackTrace.push_back(trace);
    }

    if (s)
    {
        free(s);
    }
#endif

}

const std::string& ExceptionInternal::getMessage() const
{
    return m_message;
}

int ExceptionInternal::getErrorCode() const
{
    return m_error_code;
}


const std::vector<std::string> ExceptionInternal::getStackTrace() const
{
    return m_stackTrace;
}

void ExceptionInternal::printStackTrace()
{
    if (m_line >= 0)
    {
        fprintf(stderr, "Exception raised in [%s:%d] %s(): %s\n",
                m_file.c_str(), m_line, m_function.c_str(), m_message.c_str());
    }
    else
    {
        fprintf(stderr, "Exception: %s\n", m_message.c_str());
    }

#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    for (size_t i = 0; i < m_stackTrace.size(); i++)
    {
        std::string trace = m_stackTrace.at(i);
        fprintf(stderr, "%s %lld: %s\n", TRACE_TAG, (long long)i,
                trace.c_str());
        fflush(stderr);
    }
#endif // __CYGWIN__
}

} // namespace
