/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of libcommchannel.

    libcommchannel is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libcommchannel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libcommchannel. If not, see <http://www.gnu.org/licenses/>.
*/

#include "include/commchannel.h"
#include "include/exceptions.h"

#include "exceptions_int.h"
#include "commchannel_int.h"

namespace CC
{

CommunicationData::CommunicationData(CommunicationDataID_t dataid,
                                   unsigned char *data, uint32_t size)
{
    m_internal = boost::shared_ptr<CommunicationDataInternal>(
                new CommunicationDataInternal(dataid, data, size));
}

const CommunicationDataID_t CommunicationData::getID() const
{
    return m_internal->getID();
}

const unsigned char *CommunicationData::getDataPtr() const
{
    return m_internal->getDataPtr();
}

uint32_t CommunicationData::getDataSize() const
{
    return m_internal->getDataSize();
}

CommunicationChannelServer::CommunicationChannelServer(int port, int maxconn)
{
    m_internal = boost::shared_ptr<CommunicationChannelServerInternal>(
                        new CommunicationChannelServerInternal(port, maxconn));
}

void CommunicationChannelServer::addCallback(CommunicationChannelCallback *cb)
{
    m_internal->addCallback(cb);
}

void CommunicationChannelServer::removeCallback(
                                            CommunicationChannelCallback *cb)
{
    m_internal->removeCallback(cb);
}

void CommunicationChannelServer::sendReply(CommunicationDataID_t id,
                                 unsigned char *data, uint32_t length)
{
    m_internal->sendReply(id, data, length);
}

CommunicationChannelClient::CommunicationChannelClient(int port)
{
    m_internal = boost::shared_ptr<CommunicationChannelClientInternal>(
                                new CommunicationChannelClientInternal(port));
}

boost::shared_ptr<CommunicationData> CommunicationChannelClient::sendData(
                                            unsigned char *data, uint32_t size)
{
    return m_internal->sendData(data, size);
}

boost::shared_ptr<CommunicationData> CommunicationChannelClient::sendData(
                                                       const std::string& data)
{
    return m_internal->sendData((unsigned char *)data.c_str(), data.size());
}

Exception::Exception(std::string message, int err, const char* file, int line,
                     const char* function)
{
    m_internal = boost::shared_ptr<ExceptionInternal>(
                    new ExceptionInternal(message, err, file, line, function));
}

Exception::Exception(std::string message)
{
    m_internal = boost::shared_ptr<ExceptionInternal>(
                                                new ExceptionInternal(message));
}

const std::string& Exception::getMessage() const
{
    return m_internal->getMessage();
}

int Exception::getErrorCode() const
{
    return m_internal->getErrorCode();
}

const std::vector<std::string> Exception::getStackTrace() const
{
    return m_internal->getStackTrace();
}

void Exception::printStackTrace()
{
    m_internal->printStackTrace();
}

} // namespace
