/* Time utility functions imported from MediaTomb project:
 
   MediaTomb - http://www.mediatomb.cc/
   Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                      Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
   
   Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                           Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                           Leonhard Wimmer <leo@mediatomb.cc>
   
   MediaTomb is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2
   as published by the Free Software Foundation.
   
   MediaTomb is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   version 2 along with MediaTomb; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "autoconfig.h"

#include <string>
#include <sys/time.h>
#include <errno.h>
#include <string.h>

#include "util.h"
#include "exceptions_int.h"

namespace CC
{

int timeutil::compareTimespecs(struct timespec *a,  struct timespec *b)
{
    if (a->tv_sec < b->tv_sec)
    {
        return 1;
    }
    if (a->tv_sec > b->tv_sec)
    {
        return -1;
    }
    if (a->tv_nsec < b->tv_nsec)
    {
        return 1;
    }
    if (a->tv_nsec > b->tv_nsec)
    {
        return -1;
    }
    return 0;
}

void timeutil::getTimespecNow(struct timespec *ts)
{
    struct timeval tv;
    int ret = gettimeofday(&tv, NULL);
    if (ret != 0)
    {
        throw _CCException("gettimeofday failed: " +
                         std::string(strerror(errno)));
    }

    ts->tv_sec = tv.tv_sec;
    ts->tv_nsec = tv.tv_usec * 1000;
}

void timeutil::getTimespecAfterMillis(long delta, struct timespec *ret,
                                      struct timespec *start)
{
    struct timespec now;
    if (start == NULL)
    {
        getTimespecNow(&now);
        start = &now;
    }
    ret->tv_sec = start->tv_sec + delta / 1000;
    ret->tv_nsec = (start->tv_nsec + (delta % 1000) * 1000000);
    if (ret->tv_nsec >= 1000000000) // >= 1 second
    {
        ret->tv_sec ++;
        ret->tv_nsec -= 1000000000;
    }
}

} // namespace
